/**
 * @file ssxml.cpp
 * @brief Wrap Microsoft XML (version 3+, best with 4.)
 * @author Simon Steele
 * @note Copyright (c) 2002 Simon Steele <s.steele@pnotepad.org>
 *
 * The license file (license.[txt|html]) describes the conditions under 
 * which this source may be modified / distributed.
 *
 * Unicode Status: Unicode Ready (tested).
 */
#include "stdafx.h"
#include <tchar.h>

#include "ssxml.h"

/////////////////////////////////////////////////////////////////////////////////////////////
// CXMLFragment
/////////////////////////////////////////////////////////////////////////////////////////////

CXMLFragment::CXMLFragment()
{
	m_DocPtr = NULL;

	m_DocPtr.CreateInstance( MSXML::CLSID_DOMDocument40 );

	if( m_DocPtr )
	{
		HRESULT hr = m_DocPtr->put_async(VARIANT_FALSE);
	}
}

CXMLFragment::CXMLFragment(IXMLDOMDocumentPtr document)
{
	m_DocPtr = document;
}

/**
 * Construct an XML DOM based on the fragment (or full document) of
 * xml passed in. If the parse is un-successful, then IsValid() will
 * return false.
 */
CXMLFragment::CXMLFragment(LPCTSTR fragment)
{
	m_DocPtr.CreateInstance( MSXML::CLSID_DOMDocument40 );

	if( m_DocPtr )
	{
		HRESULT hr = m_DocPtr->put_async(VARIANT_FALSE);

		LoadFragment(fragment);
	}
}

CXMLFragment::~CXMLFragment()
{
	// Hopefully this should free up the document
	// by dropping the reference count...
	m_DocPtr = NULL;
}

/**
 * Load a complete XML document or a fragment of well-formed XML.
 * @param fragment XML text.
 */
HRESULT CXMLFragment::LoadFragment(LPCTSTR fragment)
{
	VARIANT_BOOL isSuccessful;

	CString cs(fragment);
	
	HRESULT hr = E_FAIL;

	if(_tcslen(fragment) > 0)
	{
		hr = m_DocPtr->loadXML(cs.AllocSysString(), &isSuccessful);
	}
	
	return hr;	
}

/**
 * Return a CXMLNode which forms the document root. This
 * will almost always be #document, the first real node will
 * come from CXMLNode::GetFirstChild().
 */
CXMLNode CXMLFragment::GetRootItem()
{
	ASSERT(m_DocPtr != NULL);

	IXMLDOMNodePtr spRootNode = m_DocPtr;
	
	return CXMLNode(spRootNode);
}

/**
 * @return A IXMLDOMNodeList from a tagname
 */
bool CXMLFragment::GetElementByTagName( LPCTSTR tagname, CXMLNodeList &NodeList  )
{
	ASSERT(m_DocPtr != NULL);

	CString cs(tagname);
	CComBSTR myStupidString = cs/*.AllocSysString()*/;

	HRESULT hr = m_DocPtr->getElementsByTagName( myStupidString, &NodeList.m_pIDOMNodeList );

	if (S_FALSE == hr)
	{
		return false;
	}

	return true;
}

/**
 * Get the CXMLNode which forms the document root. This will
 * almost always be a #document node. The first real node will
 * come from CXMLNode::GetFirstChild().
 * @param rootitem CXMLNode instance to become the document root.
 */
bool CXMLFragment::GetRootItem(CXMLNode& rootitem)
{
	ASSERT(m_DocPtr != NULL);

	IXMLDOMNodePtr spRootNode = m_DocPtr;
	
	rootitem.SetNodePtr(spRootNode);

	return true;
}

/**
 * @return Returns false if there is content, or there was a parse error.
 */
bool CXMLFragment::IsValid()
{
	if(NULL == m_DocPtr)
		return false;

	IXMLDOMParseErrorPtr spError;

	m_DocPtr->get_parseError(&spError);

	long err;
	
	HRESULT hr = spError->get_errorCode(&err);
	

	if(FAILED(hr) || (SUCCEEDED(hr) && err == 0))
		return true;

	return false;
}

/**
 * If there was a parse error then this function will return details 
 * about it, else no details will be set.
 * @param errorcode Error code returned by the parser.
 * @param line Line number on which the error was found.
 * @param column Offset into the error-containing line at which the error was found.
 * @param reason Textual details about the error encountered.
 */
void CXMLFragment::GetParseError(long& errorcode, long& line, long& column, CString& reason)
{
	IXMLDOMParseErrorPtr spError;

	m_DocPtr->get_parseError(&spError);

	long err;
	
	HRESULT hr = spError->get_errorCode(&err);
	
	if(FAILED(hr))
		return;		// no error!

	errorcode = err;

	spError->get_line(&err);
	line = err;

	spError->get_linepos(&err);
	column = err;

	BSTR rstr;
	spError->get_reason(&rstr);
	reason = rstr;
}

HRESULT CXMLFragment::CreateElement(LPCTSTR name, IXMLDOMNodePtr& spNode, int type)
{
	HRESULT hr;

	CString nstr(name);

	VARIANT vtype;

    vtype.vt = VT_I4;
    V_I4(&vtype) = (int)type;

	hr = m_DocPtr->createNode(vtype, nstr.AllocSysString(), NULL, &spNode);

	return hr;
}

CXMLNode CXMLFragment::CreateElement(LPCTSTR name)
{
	IXMLDOMNodePtr spRootNode = NULL;
	HRESULT hr;

	hr = CreateElement(name, spRootNode, MSXML::NODE_ELEMENT);

	if(SUCCEEDED(hr))
	{
		return CXMLNode(spRootNode);
	}
	else
		return CXMLNode(NULL);
}

CXMLNode CXMLFragment::CreateTextElement(LPCTSTR name)
{
	IXMLDOMNodePtr spRootNode = NULL;
	HRESULT hr;

	hr = CreateElement(_T(""), spRootNode, MSXML::NODE_TEXT);

	if(SUCCEEDED(hr))
	{
		CString cs(name);
		BSTR myStupidString = cs.AllocSysString();
		hr = spRootNode->put_text(myStupidString);

		return CXMLNode(spRootNode);
	}
	else
		return CXMLNode(NULL);
}

HRESULT CXMLFragment::CreateElement(LPCTSTR name, IXMLDOMNodePtr& spNode)
{
	HRESULT hr;

	hr = CreateElement(name, spNode, MSXML::NODE_ELEMENT);

	return hr;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// CXMLFile
/////////////////////////////////////////////////////////////////////////////////////////////

CXMLFile::CXMLFile()
{
	m_DocPtr = NULL;

	m_DocPtr.CreateInstance( MSXML::CLSID_DOMDocument40 );

	if( m_DocPtr )
	{
		HRESULT hr = m_DocPtr->put_async(VARIANT_FALSE);
	}
}

CXMLFile::CXMLFile(LPCTSTR filename)
{
	m_DocPtr.CreateInstance( MSXML::CLSID_DOMDocument40 );

	if( m_DocPtr )
	{
		HRESULT hr = m_DocPtr->put_async(VARIANT_FALSE);

		LoadFile(filename);
	}
	else
	{
		OutputDebugString(_T("Could not create instance of MSXML document"));
	}
}

HRESULT CXMLFile::LoadFile(LPCTSTR filename)
{
	VARIANT_BOOL isSuccessful;

	_variant_t varString = filename;
	
	HRESULT hr = E_FAIL;

	if(_tcslen(filename) > 0)
	{
		hr = m_DocPtr->load(varString, &isSuccessful);
	}
	
	return hr;	
}

HRESULT CXMLFile::SaveFile(LPCTSTR filename)
{
	ASSERT(m_DocPtr != NULL);

	HRESULT hr = E_FAIL;

	if(_tcslen(filename) > 0)
	{
		_variant_t varString = filename;
		
		hr = m_DocPtr->save(varString);
	}

	return hr;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// CXMLNode
/////////////////////////////////////////////////////////////////////////////////////////////

CXMLNode::CXMLNode()
{
	m_NodePtr = NULL;
}

CXMLNode::CXMLNode(IXMLDOMNodePtr nodeptr)
{
	m_NodePtr = nodeptr;
}

CXMLNode::CXMLNode(const CXMLNode& copy)
{
	*this = copy;
}

CXMLNode& CXMLNode::operator = (const CXMLNode& setto)
{
	m_NodePtr = setto.m_NodePtr;
	return *this;
}

CXMLNode& CXMLNode::operator = (IXMLDOMNodePtr nodeptr)
{
	m_NodePtr = nodeptr;
	return *this;
}

/**
 * This sets the CXMLNode instance to use the node
 * referred to by nodeptr.
 * @param nodeptr Node to use for future operations.
 */
void CXMLNode::SetNodePtr(IXMLDOMNodePtr nodeptr)
{
	m_NodePtr = nodeptr;
}

/**
 * @return The name of the current node as a CString.
 */
CString CXMLNode::GetName()
{
	ASSERT(m_NodePtr != NULL);

	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_nodeName(&myStupidString);

	ASSERT( SUCCEEDED(hr) );

	return CString(myStupidString);
}

/**
 * @param name CString to receive the name of the current node.
 * @return true if successful, false otherwise.
 */
bool CXMLNode::GetName(CString& name)
{
	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_nodeName(&myStupidString);

	if(SUCCEEDED(hr))
	{
		name = myStupidString;
		return true;
	}

	return false;
}

/**
 * @return Text of the current node as a CString.
 */
CString	CXMLNode::GetText()
{
	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_text(&myStupidString);

	ASSERT( SUCCEEDED(hr) );

	return CString(myStupidString);
}

/**
 * @param text CString to receive the text of the current node.
 * @return true if successful, false otherwise.
 */
bool CXMLNode::GetText(CString& text)
{
	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_text(&myStupidString);

	if(SUCCEEDED(hr))
	{
		text = myStupidString;
		return true;
	}

	return false;
}

void CXMLNode::SetText(LPCTSTR text)
{
	CString cs(text);
	BSTR myStupidString = cs.AllocSysString();

	HRESULT hr = m_NodePtr->put_text(myStupidString);

	ASSERT( SUCCEEDED(hr) );
}

/**
 * @return The XML representation of the current node and children as a CString.
 */
CString	CXMLNode::GetXML()
{
	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_xml(&myStupidString);

	ASSERT( SUCCEEDED(hr) );

	return CString(myStupidString);
}

/**
 * @param xml CString into which the xml of the current node and children will be placed.
 * @return true if successful, false otherwise.
 */
bool CXMLNode::GetXML(CString& xml)
{
	BSTR myStupidString;

	HRESULT hr = m_NodePtr->get_xml(&myStupidString);

	if(SUCCEEDED(hr))
	{
		xml = myStupidString;
		return true;
	}

	return false;
}

/**
 * @return Returns the type of the current node as an MSXML::DOMNodeType enum value.
 */
MSXML::DOMNodeType CXMLNode::GetType()
{
	ASSERT(m_NodePtr != NULL);

	MSXML::DOMNodeType type;

	m_NodePtr->get_nodeType(&type);

	return type;
}

/**
 * @return true if the current node type is MSXML::NODE_TEXT, false otherwise.
 */
bool CXMLNode::IsTextNode()
{
	return GetType() == MSXML::NODE_TEXT;
}

/**
 * @return a CXMLAttributes pointing to the attributes of the current node.
 */
CXMLAttributes CXMLNode::GetAttributes()
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMNamedNodeMapPtr spNNM;

	HRESULT hr = m_NodePtr->get_attributes(&spNNM);

	ASSERT(SUCCEEDED(hr));

	IXMLDOMDocumentPtr spDoc;
		
	hr = m_NodePtr->get_ownerDocument(&spDoc);

	ASSERT(SUCCEEDED(hr));

	return CXMLAttributes(spNNM, spDoc);
}

bool CXMLNode::GetValue(long* val, long def)
{
	ASSERT(m_NodePtr != NULL);

	_variant_t ival;
	bool ret = true;
	
	HRESULT hr = m_NodePtr->get_nodeValue(&ival);

	if(SUCCEEDED(hr))
	{
		try
		{
			*val = (long)ival;
		}
		catch ( _com_error )
		{
			// Could not convert whatever it was into a long.
			*val = def;
			ret = false;
		}
	}
	else
	{
		*val = def;
		ret = false;
	}
	return ret;
}

bool CXMLNode::GetValueI64(__int64* val, __int64 def)
{
	CString myCString;
	BSTR myStupidString;
	bool ret = true;

	HRESULT hr = m_NodePtr->get_text(&myStupidString);

	if(SUCCEEDED(hr))
	{
		myCString = myStupidString;

		try
		{
			*val = _ttoi64(myCString);
		}
		catch ( ... )
		{
			*val = (__int64)def;
			ret = false;
		}
	}
	else
	{
		*val = (__int64)def;
		ret = false;
	}

	return ret;
}

bool CXMLNode::IsValid()
{
	return m_NodePtr != NULL;
}

bool CXMLNode::AddChild(CXMLNode& childnode)
{
	ASSERT(m_NodePtr != NULL);

	if (!childnode.IsValid())
		return false;

	IXMLDOMNodePtr spNewPtr = NULL;

	HRESULT hr = m_NodePtr->appendChild(childnode.m_NodePtr, &spNewPtr);

	if(SUCCEEDED(hr) && spNewPtr != NULL)
	{
		childnode.SetNodePtr(spNewPtr);
		return true;
	}
	
	return false;
}

CXMLNode CXMLNode::AddChild(IXMLDOMNodePtr& domnode)
{
	ASSERT(m_NodePtr != NULL);
	ASSERT(domnode != NULL);

	IXMLDOMNodePtr spNewPtr = NULL;
	
	HRESULT hr = m_NodePtr->appendChild(domnode, &spNewPtr);

	if(SUCCEEDED(hr) && spNewPtr != NULL)
	{
		return CXMLNode(spNewPtr);
	}

	// If we get here, we failed... :(
	return CXMLNode(NULL);
}

CXMLNode CXMLNode::AddChild(LPCTSTR name)
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMDocumentPtr	spDoc		= NULL;
	IXMLDOMNodePtr		spNode		= NULL;
	IXMLDOMNodePtr		spNewPtr	= NULL;
	
	HRESULT hr;

	hr = m_NodePtr->get_ownerDocument(&spDoc);

	if(SUCCEEDED(hr))
	{
		// Cheat and just use the CXMLFragment methods for doing this...
		
		hr = CXMLFragment(spDoc).CreateElement(name, spNode);

		if(SUCCEEDED(hr))
		{
			hr = m_NodePtr->appendChild(spNode, &spNewPtr);

			if(SUCCEEDED(hr))
			{
				return CXMLNode(spNewPtr);
			}
		}
	}

	// If we get here, we haven't succeeded... :(
	return CXMLNode(NULL);
}

CXMLNode CXMLNode::AddComment(LPCTSTR comment)
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMDocumentPtr	spDoc	= NULL;
	IXMLDOMCommentPtr	spCmt	= NULL;

	HRESULT hr = m_NodePtr->get_ownerDocument(&spDoc);

	if(SUCCEEDED(hr))
	{
		CString cs(comment);

		hr = spDoc->createComment(cs.AllocSysString(), &spCmt);

		if(SUCCEEDED(hr))
		{
			return AddChild((IXMLDOMNodePtr)spCmt);
		}
	}

	// If we get here, we haven't succeeded... :(
	return CXMLNode(NULL);
}

// Tree Navigation Functions Group ::

/**
 * @return true if node has children, false otherwise.
 */
bool CXMLNode::HasChildren()
{
	ASSERT(m_NodePtr != NULL);

	VARIANT_BOOL hasKiddies;

	m_NodePtr->hasChildNodes(&hasKiddies);

	return hasKiddies != 0;
}

/**
 * @return A CXMLNode constructed for the first child node.
 */
CXMLNode CXMLNode::GetFirstChild()
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMNodePtr child;
	
	m_NodePtr->get_firstChild(&child);

	return CXMLNode(child);
}

/**
 * @return A CXMLNode constructed for the first sibling node.
 */
CXMLNode CXMLNode::GetNextSibling()
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMNodePtr sibling;

	m_NodePtr->get_nextSibling(&sibling);

	return CXMLNode(sibling);
}

bool CXMLNode::GetNextSibling(CXMLNode& node)
{
	ASSERT(m_NodePtr != NULL);

	IXMLDOMNodePtr sibling;

	HRESULT hr = m_NodePtr->get_nextSibling(&sibling);

	if (S_FALSE == hr)
	{
		return false;
	}

	node.SetNodePtr(sibling);

	return true;
}

/**
 * @return A IXMLDOMNodeList from a tagname
 */
bool CXMLNode::GetElementsByTagName( LPCTSTR tagname, CXMLNodeList &NodeList )
{
	ASSERT(m_NodePtr != NULL);

	CString cs(tagname);
	CComBSTR myStupidString = cs;

	HRESULT hr = m_NodePtr->selectNodes( myStupidString, &NodeList.m_pIDOMNodeList );

	if (S_FALSE == hr)
	{
		return false;
	}

	return true;
}

CString	CXMLNode::Transform(CXMLFragment& stylesheet)
{
	ASSERT(m_NodePtr != NULL);

	BSTR buf;
	HRESULT hr;
	try
	{
		hr = m_NodePtr->transformNode(stylesheet.m_DocPtr, &buf);
	}
	catch(_com_error &e)
	{
		printf(e.ErrorMessage());
	}

	if(S_FALSE == hr)
	{
		throw hr;
	}

	CString ret = buf;
	::SysFreeString(buf);

	return ret;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// CXMLAttributes
/////////////////////////////////////////////////////////////////////////////////////////////

CXMLAttributes::CXMLAttributes()
{
	m_spAttrs = NULL;
}

CXMLAttributes::CXMLAttributes(IXMLDOMNamedNodeMapPtr attrs, IXMLDOMDocumentPtr doc)
{
	SetAttributes(attrs);
	SetDocument(doc);
}

CXMLAttributes::CXMLAttributes(const CXMLAttributes& copy)
{
	*this = copy;
}

CXMLAttributes& CXMLAttributes::operator = (const CXMLAttributes& other)
{
	m_spAttrs = other.m_spAttrs;
	m_spDoc = other.m_spDoc;

	return *this;
}

/**
 * @param attrs MSXML::IXMLDOMNamedNodeMapPtr representing the attributes of a node.
 */
void CXMLAttributes::SetAttributes(IXMLDOMNamedNodeMapPtr attrs)
{
	m_spAttrs = attrs;
}

/**
 * @param doc MSXML::IXMLDOMDocumentPtr representing the related document object.
 */
void CXMLAttributes::SetDocument(IXMLDOMDocumentPtr doc)
{
	m_spDoc = doc;
}

/**
 * @return The number of attributes represented by this object.
 */
int CXMLAttributes::GetCount()
{
	ASSERT(m_spAttrs != NULL);

	long len;

	m_spAttrs->get_length(&len);

	return len;
}

/**
 * @return MSXML::IXMLDOMNodePtr for use in a CXMLNode if index is a valid attribute.
 */
IXMLDOMNodePtr CXMLAttributes::operator [] (int index)
{
	ASSERT(m_spAttrs != NULL);

	if( index < 0 || index > (GetCount()-1) )
		return NULL;

	IXMLDOMNodePtr spNode;

	m_spAttrs->get_item(index, &spNode);

	return spNode;
}

/**
 * @return CXMLNode representing value. Asserts on fail.
 */
CXMLNode CXMLAttributes::Get(LPCTSTR name)
{
	ASSERT(m_spAttrs != NULL);

	BSTR bstrAttributeName = MakeBSTR(name);

	IXMLDOMNodePtr spNamedItem;

	HRESULT hr = m_spAttrs->getNamedItem(bstrAttributeName, &spNamedItem);

	ASSERT(SUCCEEDED(hr));

	return CXMLNode(spNamedItem);
}

/**
 * @return string value of the named attribute.
 */
bool CXMLAttributes::Get(LPCTSTR name, CString& value)
{
	ASSERT(m_spAttrs != NULL);

	BSTR bstrAttributeName = MakeBSTR(name);

	IXMLDOMNodePtr	spNamedItem;

	HRESULT hr = m_spAttrs->getNamedItem(bstrAttributeName, &spNamedItem);

	if(SUCCEEDED(hr) && spNamedItem != NULL)
	{
		// Will this work???
		CXMLNode(spNamedItem).GetText(value);

		return true;
	}

	return false;
}

bool CXMLAttributes::Get(LPCTSTR name, long* value)
{
	ASSERT(m_spAttrs != NULL);

	BSTR bstrAttributeName = MakeBSTR(name);

	IXMLDOMNodePtr	spNamedItem;

	HRESULT hr = m_spAttrs->getNamedItem(bstrAttributeName, &spNamedItem);

	if(SUCCEEDED(hr) && spNamedItem != NULL)
	{
		return CXMLNode(spNamedItem).GetValue(value);
	}

	return false;
}

bool CXMLAttributes::GetInt64(LPCTSTR name, __int64* value)
{
	ASSERT(m_spAttrs != NULL);

	BSTR bstrAttributeName = MakeBSTR(name);

	IXMLDOMNodePtr spNamedItem;

	HRESULT hr = m_spAttrs->getNamedItem(bstrAttributeName, &spNamedItem);

	if(SUCCEEDED(hr) && spNamedItem != NULL)
	{
		// Will this work???
		CXMLNode(spNamedItem).GetValueI64(value);

		return true;
	}

	return false;
}

bool CXMLAttributes::Set(LPCTSTR name, long value)
{
	ASSERT(m_spAttrs != NULL);
	ASSERT(m_spDoc != NULL);

	IXMLDOMAttributePtr spAttr;
	CString valname(name);
	HRESULT hr;
	
	hr = m_spDoc->createAttribute(valname.AllocSysString(), &spAttr);

	if(SUCCEEDED(hr))
	{
		VARIANT vtype;

		vtype.vt = VT_I4;
		V_I4(&vtype) = value;

		spAttr->put_nodeValue(vtype);

		IXMLDOMNodePtr spNAttr;

		m_spAttrs->setNamedItem(spAttr, &spNAttr);

		return true;
	}

	return false;
}

bool CXMLAttributes::Set(LPCTSTR name, LPCTSTR value)
{
	ASSERT(m_spAttrs != NULL);
	ASSERT(m_spDoc != NULL);

	IXMLDOMAttributePtr spAttr;
	CString valname(name);
	HRESULT hr;
	
	hr = m_spDoc->createAttribute(valname.AllocSysString(), &spAttr);

	if(SUCCEEDED(hr))
	{
		_variant_t varString = value;

		spAttr->put_nodeValue(varString);

		IXMLDOMNodePtr spNAttr;

		m_spAttrs->setNamedItem(spAttr, &spNAttr);

		return true;
	}

	return false;
}

/**
 * @return Allocated bstr representing text
 */
BSTR CXMLAttributes::MakeBSTR(LPCTSTR text)
{
	#ifdef UNICODE
	BSTR bstrVal = ::SysAllocString(text);
	#else
	wchar_t *str = new wchar_t[strlen(text)+1];
	mbstowcs(str, text, strlen(text)+1);
	BSTR bstrVal = ::SysAllocString(str);
	delete [] str;
	#endif

	return bstrVal;
}

/**
 * @return validity test for attribute
 */
bool CXMLAttributes::IsValid()
{
	return m_spAttrs != NULL;
}


/**
 * @return validity test for CXMLNodeLIst
 */
bool CXMLNodeList::IsValid()
{
	if(NULL == m_pIDOMNodeList)
		return false;
	else
		return true;
}


/**
 * @how many items in the nodelist ?
 */
bool CXMLNodeList::GetLength( long &value )
{
	HRESULT hr;
	
	ASSERT(m_pIDOMNodeList != NULL);

	hr =  m_pIDOMNodeList->get_length(&value);
	
	if( SUCCEEDED(hr) )
	{
		return true;
	}
	else
	{
		return false;
	}

}

bool CXMLNodeList::Reset()
{
	HRESULT hr;
	
	ASSERT(m_pIDOMNodeList != NULL);

	hr =  m_pIDOMNodeList->reset();
	
	if( SUCCEEDED(hr) )
	{
		return true;
	}
	else
	{
		return false;
	}

}

bool CXMLNodeList::GetItem( int pos, CXMLNode &node )
{
	HRESULT hr;
	
	ASSERT(m_pIDOMNodeList != NULL);

	IXMLDOMNodePtr spNode = NULL;

	hr =  m_pIDOMNodeList->get_item( pos, &spNode );
	
	if( SUCCEEDED(hr) )
	{
		node = CXMLNode(spNode);

		return true;
	}
	else
	{
		return false;
	}

}

MSXML::IXMLDOMNodeList *CXMLNodeList::GetSafeNodeList()
{
	ASSERT ( m_pIDOMNodeList==NULL );
	
	return m_pIDOMNodeList; 
	
}

/// No error checking, be careful!
CXMLNode CXMLNodeList::operator [] (int pos)
{
	CXMLNode ret;
	GetItem(pos, ret);
	return ret;
}