/**
 * @file ssxml.h
 * @brief Wrap Microsoft XML (version 3+, best with 4.)
 * @author Simon Steele
 * @note Copyright (c) 2002 Simon Steele <s.steele@pnotepad.org>
 *
 * The license file (license.[txt|html]) describes the conditions under 
 * which this source may be modified / distributed.
 *
 * Unicode Status: Unicode Ready (tested).
 */
#ifndef __MSXML_INTERFACE_IMPORT_H__
#define __MSXML_INTERFACE_IMPORT_H__

#include <atlbase.h>

#import "msxml4.dll" named_guids raw_interfaces_only
//#import "msxml.dll" rename_namespace("MSXML")

using namespace MSXML2;

#define MSXML MSXML2

class CXMLAttributes;
class CXMLFragment;
class CXMLNodeList;

/**
 * Wrapper class for IXMLDOMNode using smart pointers
 * to ensure minimal (hopefully no) leakage.
 */
class CXMLNode
{
	public:
		CXMLNode();
		CXMLNode(IXMLDOMNodePtr	nodeptr);
		CXMLNode(const CXMLNode& copy);

		void SetNodePtr(IXMLDOMNodePtr nodeptr);

		CXMLNode& operator = (const CXMLNode& setto);
		CXMLNode& operator = (IXMLDOMNodePtr nodeptr);

		bool				IsValid();

		MSXML2::DOMNodeType	GetType();
		bool				IsTextNode();

		CString				GetName();
		bool				GetName(CString& name);

		CString				GetText();
		bool				GetText(CString& text);
		void				SetText(LPCTSTR text);

		CString				GetXML();
		bool				GetXML(CString& xml);

		CXMLAttributes		GetAttributes();

		bool				GetValue(long* val, long def = 0);
		
		bool				GetValueI64(__int64* val, __int64 def = (__int64)0);

		bool				AddChild(CXMLNode& childnode);
		CXMLNode			AddChild(LPCTSTR name);
		CXMLNode			AddChild(IXMLDOMNodePtr& domnode);
		CXMLNode			AddComment(LPCTSTR comment);

		CString				Transform(CXMLFragment& stylesheet);

		/**@name Tree Navigation Functions */
		//@{
		bool				HasChildren();
		CXMLNode			GetFirstChild();
		CXMLNode			GetNextSibling();
		bool				GetNextSibling(CXMLNode& node);
		bool				GetElementsByTagName( LPCTSTR tagname, CXMLNodeList &NodeList );
		//@}

	protected:
		IXMLDOMNodePtr	m_NodePtr;
};

// Disable warnings about long jimmy names...
#pragma warning(disable:4786)

/**
 * Wrapper class for IXMLDOMNamedNodeMapPtr using smart pointers.
 * This class is designed to make it easy to get the value of attributes
 * of items.
 */
class CXMLAttributes
{
	public:
		CXMLAttributes();
		CXMLAttributes(IXMLDOMNamedNodeMapPtr attrs, IXMLDOMDocumentPtr doc);
		CXMLAttributes(const CXMLAttributes& copy);

		CXMLAttributes& operator = (const CXMLAttributes& other);

		void		SetAttributes(IXMLDOMNamedNodeMapPtr attrs);
		void		SetDocument(IXMLDOMDocumentPtr doc);

		int			GetCount();
		IXMLDOMNodePtr	operator [] (int index);

		
		bool		Get(LPCTSTR name, CString& value);
		bool		Get(LPCTSTR name, long* value);

		bool		GetInt64(LPCTSTR name, __int64* value);
	
		CXMLNode	Get(LPCTSTR name);

		bool		Set(LPCTSTR name, long value);
		bool		Set(LPCTSTR name, LPCTSTR value);

		bool		IsValid();

	protected:
		IXMLDOMNamedNodeMapPtr	m_spAttrs;

		IXMLDOMDocumentPtr		m_spDoc;

		BSTR MakeBSTR(LPCTSTR text);
};

class CXMLNodeList
{
	// DOH
	public:
		MSXML2::IXMLDOMNodeList *m_pIDOMNodeList;
	
	
	public:
		CXMLNodeList()			{	m_pIDOMNodeList = NULL; }
		virtual ~CXMLNodeList() { 
									if( IsValid() )
									{
										m_pIDOMNodeList->Release();
										m_pIDOMNodeList = NULL;
									}
								};

		bool		GetLength( long &value );
		bool		Reset();
		bool		GetItem( int pos, CXMLNode &node );

		CXMLNode	operator [] (int pos);

		MSXML2::IXMLDOMNodeList *GetSafeNodeList();

		bool		IsValid();




};

/**
 * Re-Start of the XML Dom wrapper classes. We start with 
 * CXMLFragment which is designed to parse a string of XML.
 * The Root node can then be gained through GetRootNode().
 */
class CXMLFragment
{
	friend class CXMLNode;

	protected:
		IXMLDOMDocumentPtr   m_DocPtr;

	public:
		CXMLFragment();
		CXMLFragment(LPCTSTR fragment);
		CXMLFragment(IXMLDOMDocumentPtr document);

		virtual ~CXMLFragment();

		HRESULT LoadFragment(LPCTSTR fragment);

		bool IsValid();
		void GetParseError(long& errorcode, long& line, long& column, CString& reason);

		CXMLNode GetRootItem();
		bool GetRootItem(CXMLNode& rootitem);

		HRESULT CreateElement(LPCTSTR name, IXMLDOMNodePtr& spNode, int type);

		CXMLNode CreateElement(LPCTSTR name);
		HRESULT CreateElement(LPCTSTR name, IXMLDOMNodePtr& spNode);
		
		CXMLNode CreateTextElement(LPCTSTR text);

		bool GetElementByTagName( LPCTSTR tagname, CXMLNodeList &pNodeList );
		
};

class CXMLFile : public CXMLFragment
{
	public:
		CXMLFile();
		CXMLFile(LPCTSTR filename);

		HRESULT LoadFile(LPCTSTR filename);
		HRESULT SaveFile(LPCTSTR filename);
};

#endif